#!/bin/bash
# vim: set et ts=4 sw=0:
set -e

die() {
    echo "${@}" >&2
    exit 2
}

# 额外添加识别特定命令行选项
declare +x keepCliAppJvmArg
declare opts_file=opts
for e in "${@}" ; do
    case "${e}" in
    --daemon|--quit)
        keepCliAppJvmArg=1
        ;;
    *)
        ;;
    esac
done

if test -z "${APP_HOME}" ; then
    APP_HOME="$(cd "$(dirname "${0}")"/.. && pwd)"
fi

declare +x APP_NAME
if test -z "${APP_NAME}" ; then
    APP_NAME="$(basename "${0}" .sh)"
fi

# find java
declare +x JAVA
while test -z "${JAVA}" ; do
    # try bundled "jre/" first.
    if test -x "${APP_HOME}/jre/bin/sentinel" && "${APP_HOME}/jre/bin/sentinel" --version >/dev/null 2>&1 ; then
        JAVA="${APP_HOME}/jre/bin/sentinel"
        break
    fi
    if test -x "${APP_HOME}/jre/bin/java" && "${APP_HOME}/jre/bin/java" -version >/dev/null 2>&1 ; then
        JAVA="${APP_HOME}/jre/bin/java"
        break
    fi

    # try JAVA_HOME
    #TODO check java -version ?
    if test -n "${JAVA_HOME}" -a -x "${JAVA_HOME}/bin/java" && "${JAVA_HOME}/bin/java" -version >/dev/null 2>&1 ; then
        JAVA="${JAVA_HOME}/bin/java"
        JAVA_="$(realpath "${JAVA}" 2>/dev/null || true)"
        if test -n "${JAVA_}" ; then
            JAVA="${JAVA_}"
        fi
        unset JAVA_
        break
    fi

    # fallback
    if java -version >/dev/null 2>&1 ; then
        JAVA=java
        break
    else
        die "FATAL: java NOT available !!"
    fi
done

# check running env.

export CLASSPATH

if test -d "${APP_HOME}/share/maven-repo/" -a -f "${APP_HOME}/etc/classpath.txt" ; then
    # running from installed env.
    etc="${APP_HOME}/etc"
    repo="${APP_HOME}/share/maven-repo"
    lib="${APP_HOME}/lib"

    # use resources as the first CLASSPATH item.
    if test -d "${APP_HOME}/etc/resources/" ; then
        CLASSPATH="${APP_HOME}/etc/resources"
    else
        CLASSPATH=
    fi

else
    # running from building tree.
    etc="${APP_HOME}/target/etc"
    repo="${APP_HOME}/target/repo"
    lib="${APP_HOME}/target/lib"

    # use project "classes/" as the first CLASSPATH item, which contains resources.
    CLASSPATH="${APP_HOME}/target/classes"
fi

# build CLASSPATH
# NOTE: don't use pipe here, which would make "while" run in a subshell.
while read path ; do
    if test -z "${path}" ; then
        continue
    fi

    # ensure path exists ?
    path="${repo}/${path}"
    if ! test -e "${path}" ; then
        ls -ld "${path}" -F
    fi

    if test -z "${CLASSPATH}" ; then
        sep=
    else
        sep=:
    fi

    CLASSPATH="${CLASSPATH}${sep}${path}"
done < "${etc}/classpath.txt"

# build jrunCmd.
# parse as bash array
declare -a jrunCmd=()
eval "jrunCmd=( $(cat "${etc}/args.txt") "$'\n'" )"

# 记录命令行应用参数下标
cliAppArgIndex="${#jrunCmd[@]}"
# 追加命令行参数
jrunCmd=( "${jrunCmd[@]}" "${@}" )

# 处理并更新 jrunCmd 数组保存的 JVM 参数和应用参数, 将 -D* -J* 等应用参数转换为 jvm 参数.
# 注意: first none "-*" arg is app arg
declare -a jrunCmd
# keepCliAppJvmArg 非空时, 将命令行上的 -D* -J* 等需转换为 jvm 参数的应用参数同时也保留透传给应用.
declare -- keepCliAppJvmArg
# 命令行应用参数在 jrunCmd 数组中的起始下标, 若未设置则 keepCliAppJvmArg 将无效.
declare -i cliAppArgIndex
updateJvmArgv() {
	local -- keepAppJvmArg
	if test -n "${cliAppArgIndex}" ; then
		keepAppJvmArg="${keepCliAppJvmArg}"
	else
		# cliAppArgIndex 未设置, 禁用 keepAppJvmArg
		keepAppJvmArg=
	fi
	# 始终设置 keepArgIdx 数值, 即使 cliAppArgIndex 未设置, 避免检查 keepArgIdx 条件时语法错误.
	# 兼容 cliAppArgIndex 配置错误的情况.
	local -i keepArgIdx
	(( keepArgIdx = cliAppArgIndex )) || true

	# split jvmArgv and appArgv
	local jvmArgv=()
	local appArgv=()
	local argvPtr=jvm
	local -i appJvmArg=0
	# skip ${jrunCmd[0]} , 有效 `idx` 从 1 开始 (每轮循环开始时 +1) .
	local -i idx=0
	for arg in "${jrunCmd[@]:1}" ; do
		(( idx += 1 ))
		if test "${argvPtr}" = jvm ; then
			# first none "-*" arg is app arg
			if [[ "${arg}" != -* ]] ; then
				argvPtr=app
			fi
		fi

		appJvmArg=0
		if test "${argvPtr}" = jvm ; then
			jvmArgv=( "${jvmArgv[@]}" "${arg}" )

		elif [[ "${arg}" = -D* ]] ; then
			# always pass -D* as jvmArgv
			appJvmArg=1
			jvmArgv=( "${jvmArgv[@]}" "${arg}" )
		elif [[ "${arg}" = -J* ]] ; then
			# app arg -J* are strip to jvmArgv
			appJvmArg=1
			jvmArgv=( "${jvmArgv[@]}" "${arg:2}" )

		else
			appArgv=( "${appArgv[@]}" "${arg}" )
		fi

		if test -n "${keepAppJvmArg}" -a "${idx}" -ge "${keepArgIdx}" -a "${appJvmArg}" -eq 1 ; then
			# 将命令行上的 -D* -J* 等需转换为 jvm 参数的应用参数同时也保留透传给应用.
			appArgv=( "${appArgv[@]}" "${arg}" )
		fi
	done

    if : > "${APP_HOME}/log/${opts_file}" 2>/dev/null ; then
        echo "-Djava.library.path=\"${lib}\"" >> "${APP_HOME}/log/${opts_file}"
        if test -n "${JPDA_OPTS}" ; then
            echo "${JPDA_OPTS}" >> "${APP_HOME}/log/${opts_file}"
        fi
        for arg in "${jvmArgv[@]}" "${appArgv[0]}" ; do
            sed -re '/([\\"]|\s)/ { s#[\\"]#\\\0#g ; s#^#"# ; s#$#"# }' <<< "${arg}" >> "${APP_HOME}/log/${opts_file}"
        done
        jrunCmd=( "${jrunCmd[0]}" @log/"${opts_file}" "${appArgv[@]:1}" )
    else
        jrunCmd=( "${jrunCmd[0]}" -Djava.library.path="${lib}" ${JPDA_OPTS} "${jvmArgv[@]}" "${appArgv[@]}" )
    fi
}

mkdir -p "${APP_HOME}"/log || die "mkdir -p ${APP_HOME}/log FAILED"
updateJvmArgv

# start app, always use ${APP_HOME} as working directory ?
cd "${APP_HOME}" || die "cd ${APP_HOME} FAILED"
# replace ${jrunCmd[0]} with ${JAVA} .
exec "${JAVA}" "${jrunCmd[@]:1}" || die "exec sentinel sidecar FAILED"
