#!/bin/bash
# vim: set noet ts=4:

# 简单设置 demo nginx 配置文件, 启动、停止 demo nginx 。

set -e

: "${DEMO_HOME:=$(cd "$(dirname "${0}")" && pwd)}"
: "${SENTINEL_HOME:=$(cd "$(dirname "${DEMO_HOME}")" && pwd)}"

LIB=
DEMO_BIN=
if test -d "${SENTINEL_HOME}/cxx/nginx-sentinel-module/" ; then
	# 开发环境, 引用 nginx-sentinel-module/ 下的 lib/ 目录
	LIB="${SENTINEL_HOME}/cxx/nginx-sentinel-module/lib"
	DEMO_BIN="${SENTINEL_HOME}/cxx/nginx-sentinel-module/bin"
else
	# 正式安装环境, 引用 "${SENTINEL_HOME}/lib"/ 目录
	LIB="${SENTINEL_HOME}/lib"
	DEMO_BIN="${DEMO_HOME}/bin"
fi

# 检查当前网络环境 。
# 查询实例元数据 region-id, 参考 https://help.aliyun.com/knowledge_detail/49122.html 。
# 查询成功则认为当前环境为阿里云 VPC 环境。
regionId=
if regionId="$( curl -sL --fail -m 1 http://100.100.100.200/latest/meta-data/region-id )" ; then
	echo "Network Environment: VPC Region ${regionId}"
else
	regionId="public"
	echo "Network Environment: Non-VPC"
fi

# 解析命令行参数。
# 特殊解析 --app=<app> --license=<license> 等参数用于配置 sentinel sidecar, 其他参数直接透传给 nginx 。

args=()
license=
sentinelArgs=()

for opt in "${@}" ; do
	case "${opt}" in
	# 特殊解析 --* 选项
	--*)
		case "${opt}" in
		--app)
			echo "ERROR: ${opt}= missing argument" >&2
			exit 1
			;;
		--app=*)
			if test -z "${opt#--app=}" ; then
				echo "ERROR: --app= missing argument" >&2
				exit 1
			fi
			sentinelArgs=( "${sentinelArgs[@]}" "${opt}" )
			;;

		--ns)
			echo "ERROR: ${opt}= missing argument" >&2
			exit 1
			;;
		--ns=*)
			if test -z "${opt#--ns=}" ; then
				echo "ERROR: ${opt} missing argument" >&2
				exit 1
			fi
			sentinelArgs=( "${sentinelArgs[@]}" "${opt}" )
			;;

		--license)
			echo "ERROR: --license= missing argument" >&2
			exit 1
			;;
		--license=*)
			if test "${regionId}" = "public" ; then
				license="${opt#--license=}"
				if test -z "${opt#--license=}" ; then
					echo "ERROR: --license= missing argument" >&2
					exit 1
				fi
				sentinelArgs=( "${sentinelArgs[@]}" "${opt}" )
			else
				echo "WARN: VPC ignore option: ${opt}" >&2
			fi
			;;

		--agw-type)
			echo "ERROR: ${opt}= missing argument" >&2
			exit 1
			;;
		--agw-type=*)
			if test -z "${opt#--agw-type=}" ; then
				echo "ERROR: ${opt} missing argument" >&2
				exit 1
			fi
			sentinelArgs=( "${sentinelArgs[@]}" "${opt}" )
			;;

		--agw-addr)
			echo "ERROR: ${opt}= missing argument" >&2
			exit 1
			;;
		--agw-addr=*)
			if test -z "${opt#--agw-addr=}" ; then
				echo "ERROR: ${opt} missing argument" >&2
				exit 1
			fi
			sentinelArgs=( "${sentinelArgs[@]}" "${opt}" )
			;;

		--*)
			echo "ERROR: Unkown option: ${opt}" >&2
			exit 1
			;;
		esac
		;;

	# 其他选项全部透传给 nginx
	*)
		args=( "${args[@]}" "${opt}" )
		;;
	esac
done

# 剔除 "--*" 特殊选项后, 重置命令行参数。
set -- "${args[@]}"

# 设置 nginx 配置文件.
# 首次启动 (nginx 配置文件不存在) 时必须先设置 nginx 配置文件,
# 否则指定了 --app=<app> 或 --license=<license> 时才重新设置 nginx 配置文件。
if test ! -e "${DEMO_HOME}/etc/nginx.conf" -o "${#sentinelArgs[@]}" -gt 0 ; then
	# 如果未设置 --app=<app> , 应用名默认为 "demo"
	: "${app:=demo}"

	if test "${regionId}" = "public" ; then
		# 非阿里云 VPC 环境, 提示公网环境需要设置 --license=<license>
		if test -z "${license}" ; then
			echo "WARN: missing --license=<license> , Public internet requires license .
WARN: You can get your license from AHAS Console: https://ahas.console.aliyun.com/#/SystemGuard?regionId=${regionId}&ns=default" >&2
		fi
	fi
	echo "Config sentinel_sidecar_run: ${sentinelArgs[*]}"

	sedUser=()
	if test "$(id -u)" -eq 0 ; then
		sedUser=( -e "/^#user / s/^#//" )
	fi

	# TODO sed 表达式最好应转义表示 "${sentinelArgs[*]}"
	sed -r -e "s#@NGX_GROUP@#$(id -gn nobody)#g" \
		-e "s#@DEMO_HOME@#${DEMO_HOME}#g ; s#@LIB@#${LIB}#g ; s#@SENTINEL_HOME@#${SENTINEL_HOME}#g" \
		-e "/^sentinel_sidecar_run/ s#--app=.*;\$#${sentinelArgs[*]};#" \
		"${sedUser[@]}" \
		"${DEMO_HOME}"/etc/nginx.conf.in > "${DEMO_HOME}"/etc/nginx.conf
fi

echo "AHAS Console: https://ahas.console.aliyun.com/#/SystemGuard?regionId=${regionId}&ns=default"

# 调用 nginx 命令
cd "${DEMO_HOME}"
exec "${DEMO_BIN}"/nginx -p "${DEMO_HOME}" -c "${DEMO_HOME}"/etc/nginx.conf "${@}"
